################################################################################
#
# Copyright (c) 2002-2005, Benjamin Saller <bcsaller@ideasuite.com>, and
#                              the respective authors. All rights reserved.
# For a list of Archetypes contributors see docs/CREDITS.txt.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice, this
#   list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
# * Neither the name of the author nor the names of its contributors may be used
#   to endorse or promote products derived from this software without specific
#   prior written permission.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
################################################################################

from Testing.ZopeTestCase import user_password
from Products.Five.testbrowser import Browser

from Products.Archetypes.tests.atsitetestcase import ATSiteTestCase
from Products.Archetypes.tests.atsitetestcase import ATFunctionalSiteTestCase
from Products.Archetypes.tests.utils import makeContent


from Products.CMFCore.utils import _checkPermission as checkPerm
from Products.CMFCore import permissions


class TestPermissions(ATSiteTestCase):
    demo_types = ['DDocument', 'SimpleType', 'SimpleFolder',
                  'Fact', 'ComplexType']

    def afterSetUp(self):
        ATSiteTestCase.afterSetUp(self)
        # install AT within portal
        self.login()
        self.demo_instances = []
        for t in self.demo_types:
            # XXX: Fails with "Unauthorized" exception from
            #      CMFDefault/DiscussionTool.py:84, in overrideDiscussionFor
            #
            #      Note that BaseObject.initializeArchetype has a bare except
            #      that prints out the error instead of letting it through, so
            #      that there is no exception when running the test.
            inst = makeContent(self.folder, portal_type=t, id=t)
            self.demo_instances.append(inst)

    def testPermissions(self):
        for content in self.demo_instances:
            # XXX: Strangely enough we have correct permissions here, but not so
            #      in initializeArchetype
            self.failUnless(checkPerm(permissions.View, content))
            self.failUnless(checkPerm(permissions.AccessContentsInformation, content))
            self.failUnless(checkPerm(permissions.ModifyPortalContent, content))

    def testRendering(self):
        # Attempt to call each object and make sure it presents a rendered
        # html view
        for content in self.demo_instances:
            self.failUnless(isinstance(content(), basestring))
            self.failUnless(content().strip().startswith('<!DOCTYPE'))


class TestFTICopy(ATFunctionalSiteTestCase):
    """Test for http://dev.plone.org/plone/ticket/6734: Cannot filter
    Addable Types with folderish FTI in portal_types.
    """
    
    def test6734(self):
        self.loginAsPortalOwner()

        # We start off by copying the existing SimpleFolder type to
        # our own type 'MySimpleFolder'.  For this type, we set the
        # SimpleFolder type to be the sole allowed content type.
        types = self.portal.portal_types
        types.manage_pasteObjects(types.manage_copyObjects(['SimpleFolder']))
        types.manage_renameObjects(['copy_of_SimpleFolder'], ['MySimpleFolder'])
        my_type = types['MySimpleFolder']
        attrs = dict(allowed_content_types=('SimpleFolder',),
                     filter_content_types=True,
                     portal_type='MySimpleFolder',
                     title='MySimpleFolder')
        my_type.__dict__.update(attrs)

        browser = Browser()
        browser.addHeader('Authorization',
                          'Basic %s:%s' % ('portal_owner', user_password))
        browser.open(self.folder.absolute_url())
        browser.getLink('Add new').click()
        browser.getControl('MySimpleFolder').click()
        browser.getControl('Add').click()

        browser.getControl('Title').value = 'My dope folder'
        browser.getControl('Save').click()
        self.failUnless('Changes saved.' in browser.contents)
        self.failUnless('My dope folder' in browser.contents)


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestPermissions))
    suite.addTest(makeSuite(TestFTICopy))
    return suite



from setuptools import setup, find_packages

version = '1.5.11'

setup(name='Products.Archetypes',
      version=version,
      description="Archetypes is a developers framework for rapidly "
                  "developing and deploying rich, full featured content "
                  "types within the context of Zope/CMF and Plone.",
      long_description=open("README.txt").read(),
      classifiers=[
        "Framework :: Plone",
        "Framework :: Zope2",
        "Programming Language :: Python",
        ],
      keywords='Archetypes Plone CMF Zope',
      author='Benjamin Saller and others',
      author_email='plone-developers@lists.sourceforge.net',
      url='http://plone.org/',
      license='BSD',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['Products'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'Products.Marshall',
          'Products.MimetypesRegistry',
          'Products.PortalTransforms',
          'Products.validation',
      ],
      )



"""
Graphviz local object referencs, allows any refrerenceable object to
produce a graph and a client side map. When we can export this as SVG
(and expect clients to handle it) it will be much easier to style to
the look of the site.

Inspired by code from Andreas Jung
"""


from urllib import unquote
from cStringIO import StringIO
from popen2 import popen2
from config import HAS_GRAPHVIZ, GRAPHVIZ_BINARY

def obj2id(obj):
    """ convert an issue to an ID """
    str = obj.absolute_url(1)
    return str2id(str)

def str2id(str):
    id = unquote(str)
    id = id.replace('-', '_')
    id = id.replace('/', '_')
    id=  id.replace(' ', '_')
    id=  id.replace('.', '_')
    return id



class Node:
    """ simple node class """

    def __init__(self, inst):
        self.id = obj2id(inst)
        self.url = inst.absolute_url()
        self.uid = inst.UID()
        self.title = inst.title_or_id()
        self.text = '%s: %s' % (inst.getId(), inst.Title())

    def __str__(self):
        return self.id

    __repr__ = __str__

class Edge:
    """ simple edge class """

    def __init__(self, src, dest, reference):
        self.src = src
        self.dest = dest
        self.relationship = reference.relationship

    def __str__(self):
        return '%s -> %s [label="%s", href="%s/reference_graph"]' % (self.src,
                                                    self.dest,
                                                    self.relationship,
                                                    self.src.url)
    def __hash__(self):
        return hash((self.src.uid,  self.dest.uid, self.relationship))

    __repr__ = __str__

def local_reference_graph(inst):
    nodes  = {}
    graphs = { 'forward' : {},
               'backward' : {},
               }

    rc = inst.reference_catalog

    references = rc.getReferences(inst)
    back_references = rc.getBackReferences(inst)

    node = Node(inst)
    nodes[inst.UID()] = node

    for ref in references:
        tob = ref.getTargetObject()
        target = Node(tob)
        if tob.UID() not in nodes:
            nodes[tob.UID()] = target

        e = Edge(node, target, ref)
        graphs['forward'].setdefault(ref.relationship, []).append(e)

    for ref in back_references:
        sob = ref.getSourceObject()
        source = Node(sob)
        if sob.UID() not in nodes:
            nodes[sob.UID()] = source

        e = Edge(source, node, ref)
        graphs['backward'].setdefault(ref.relationship, []).append(e)

    return graphs

# typo, but keep API
local_refernece_graph = local_reference_graph

def build_graph(graphs, inst):
    fp = StringIO()
    print >>fp, 'digraph G {'
    uid = inst.UID()
    seen = {}
    shown = {}
    for direction, graph in graphs.iteritems(): #forw/back
        for relationship, edges in graph.iteritems():
            rel_id = "unqualified"
            if relationship: rel_id = str2id(relationship)
            print >>fp, 'subgraph cluster_%s {' % rel_id


            for e in iter(edges):
                for n in e.src, e.dest:
                    if n not in seen:
                        seen[n] = 1
                        print >>fp, '\t%s [label="%s", href="%s"' % (n.id,
                                                                   n.title,
                                                                   n.url),
                        if uid == n.uid:
                            print >>fp, '\tstyle=filled, fillcolor=blue',
                        print >>fp, ']'

            for e in iter(edges):
                if e in shown: continue
                if direction == "forward":
                    print >>fp, '\t%s -> %s [label="%s", href="%s/reference_graph"]' % (
                    e.src,
                    e.dest,
                    e.relationship,
                    e.dest.url)
                else:
                    print >>fp, '\t%s -> %s [label="%s", href="%s/reference_graph"]' % (
                    e.src,
                    e.dest,
                    e.relationship,
                    e.src.url)
                shown[e] = e

            print >>fp, '\t}\n'

    print >>fp, "}"
    return fp.getvalue()




if HAS_GRAPHVIZ:
    def getDot(inst):
        g = local_reference_graph(inst)
        data = build_graph(g, inst)
        return data

    def get_image(inst, fmt):
        data = getDot(inst)

        stdout, stdin = popen2('%s -Gpack -T%s' % (GRAPHVIZ_BINARY, fmt))
        stdin.write(data)
        stdin.close()
        output = stdout.read()
        return output

    def get_png(inst): return get_image(inst, fmt="png")

    def get_cmapx(inst):
        data = getDot(inst)

        stdout, stdin = popen2('%s -Gpack -Tcmapx ' % GRAPHVIZ_BINARY)
        stdin.write(data)
        stdin.close()
        output = stdout.read()
        return output


else:
    def get_png(inst): return None
    def get_cmapx(inst): return None



################################################################################
#
# Copyright (c) 2002-2005, Benjamin Saller <bcsaller@ideasuite.com>, and
#                              the respective authors. All rights reserved.
# For a list of Archetypes contributors see docs/CREDITS.txt.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice, this
#   list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
# * Neither the name of the author nor the names of its contributors may be used
#   to endorse or promote products derived from this software without specific
#   prior written permission.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
################################################################################

from Products.Archetypes.tests.atsitetestcase import ATSiteTestCase
from Products.Archetypes.tests.utils import mkDummyInContext

from Products.Archetypes import PloneMessageFactory as _
from Products.Archetypes.atapi import *

class DummyDiscussionTool:
    def isDiscussionAllowedFor( self, content ):
        return False
    def overrideDiscussionFor(self, content, allowDiscussion):
        pass

MULTIPLEFIELD_LIST = DisplayList(
    (
    ('1', _(u'Option 1 : printemps')),
    ('2', unicode('Option 2 : \xc3\xa9t\xc3\xa9', 'utf-8')), # e-acute t e-acute
    ('3', u'Option 3 : automne'),
    ('4', _(u'option3', default=u'Option 3 : hiver')),
    ))

schema = BaseSchema + Schema((
    LinesField(
        'MULTIPLEFIELD',
        searchable = 1,
        vocabulary = MULTIPLEFIELD_LIST,
        widget = MultiSelectionWidget(
            i18n_domain = 'plone',
            ),
        ), 
    TextField(
        'TEXTFIELD',
        primary=True,
        ),
))

class Dummy(BaseContent):
   
    portal_discussion = DummyDiscussionTool()

    def getCharset(self):
         return 'utf-8'
         
class BaseObjectTest(ATSiteTestCase):

    def afterSetUp(self):
        ATSiteTestCase.afterSetUp(self)
        self._dummy = mkDummyInContext(Dummy, oid='dummy', context=self.portal,
                                      schema=schema)
    
    def test_searchableText(self):
        """
        Fix bug [ 951955 ] BaseObject/SearchableText and list of Unicode stuffs
        """
        dummy = self._dummy

        # Set a multiple field
        dummy.setMULTIPLEFIELD(['1','2'])
        searchable = dummy.SearchableText()

        self.failUnless(isinstance(searchable, basestring))
        self.assertEquals(searchable, '1 2 Option 1 : printemps Option 2 : \xc3\xa9t\xc3\xa9')

        dummy.setMULTIPLEFIELD(['3','4'])
        searchable = dummy.SearchableText()

        self.assertEquals(searchable, '3 4 Option 3 : automne option3')

    def test_searchableTextUsesIndexMethod(self):
        """See http://dev.plone.org/archetypes/ticket/645

        We want SearchableText to use the ``index_method`` attribute
        of fields to determine which is the accessor it should use
        while gathering values.
        """
        dummy = self._dummy
        
        # This is where we left off in the previous test
        dummy.setMULTIPLEFIELD(['1','2'])
        searchable = dummy.SearchableText()
        self.failUnless(searchable.startswith('1 2 Option 1 : printemps'))

        # Now we set another index_method and expect it to be used:
        dummy.getField('MULTIPLEFIELD').index_method = 'myMethod'
        def myMethod(self):
            return "What do you expect of a Dummy?"
        Dummy.myMethod = myMethod
        searchable = dummy.SearchableText()
        self.failUnless(searchable.startswith("What do you expect of a Dummy"))
        del Dummy.myMethod
        
    def test_authenticatedContentType(self):
        """See https://dev.plone.org/archetypes/ticket/712
        
        content_type should not be protected by a security declaration, as
        it is usually an attribute. If a security declaration *is* set (in
        BaseObject or one of it's base classes) non-anonymous access from
        protected code (guarded_getattr) will fail.
        
        """
        from AccessControl.unauthorized import Unauthorized
        from AccessControl.Permissions import view
        from AccessControl.ZopeGuards import guarded_getattr
        
        dummy = self._dummy
        dummy.manage_permission(view, ('Manager',), False)
        # dummy.content_type in a Python Script
        self.assertRaises(Unauthorized, guarded_getattr, dummy, 'content_type')
        
        self.setRoles(('Manager',))
        # dummy.content_type in a Python Script
        self.assertEqual(guarded_getattr(dummy, 'content_type'), 'text/html')
        

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(BaseObjectTest))
    return suite



""" IMPORTANT: this module is unusable before initialize is called
    this must be so because we want to make sure all products
    (eg, whatever translation service we're supposed to use)
    is already there and ready
"""
from Products.Archetypes.debug import deprecated
deprecated('generator.i18n is deprecated and will be removed in Archetypes 1.6.'
           'If your product uses PTS or Localizer use the translate method of '
           'the GlobalTranslationService instead. Usually you should simply do '
           '"from zope.i18n import translate" and use that method.')

from Products.PageTemplates.GlobalTranslationService import \
     getGlobalTranslationService, DummyTranslationService

service = None
translate = None

def translate_wrapper(domain, msgid, mapping=None, context=None,
                      target_language=None, default=None):
    """Wrapper for calling the translate() method with a fallback value."""
    res = service.translate(domain, msgid, mapping=mapping, context=context,
                            target_language=target_language,
                            default=default)

    if res is None or res is msgid:
        return default
    return res

def null_translate(domain, msgid, mapping=None, context=None,
                   target_language=None, default=None):
    return default

def initialize():
    """Must be called after Products are there and ready."""
    global service, translate
    service = getGlobalTranslationService()
    if service is DummyTranslationService:
        translate = null_translate
    elif hasattr(service, '_fallbacks'):
        # It accepts the "default" argument
        translate = service.translate
    else:
        translate = translate_wrapper

def initial_translate(domain, msgid, mapping=None, context=None,
                      target_language=None, default=None):
    initialize()
    deprecated('Archetypes.generator.i18n.translate is deprecated and will be '
               'removed in Archetypes 1.6. Please use "from zope.i18n import '
               'translate" instead.')
    return translate(domain, msgid, mapping, context, target_language, default)

translate = initial_translate



"""
AggregatedStorage for Archetypes

(C) 2004, Andreas Jung & Econtec GmbH, D-90492 Nuernberg, Germany

Released as open-source under the current Archetypes license

$Id: AggregatedStorage.py 7768 2007-03-18 23:38:08Z nouri $
"""

from time import time
from types import StringType, DictType
from threading import Lock

from Storage import Storage
from Registry import Registry

CACHE_TIMEOUT = 5  # timeout in seconds for cache entries to expire

class AggregatedStorage(Storage):
    """ Implementation of the AggregatedStorage proposal as described in http://plone.org/development/teams/developer/AggregatedStorage """

    def __init__(self, caching=0):
        self._reg_ag = Registry(StringType)  # registry for aggregators
        self._reg_dag = Registry(StringType) # registry for disaggregators
        self.cache = {}                      # map (objId, aggregator) -> (timestamp, result_dict)
        self._caching = caching
        self._lock = Lock()
        
    def __getstate__(self):
        """Override __getstate__ used for copy operations
        
        Required to fix the copy problem with the lock
        """
        state = self.__dict__
        state['_lock'] = None
        return state

    def __setstate__(self, state):
        """Override __setstate__ used for copy operations
        
        Required to fix the copy problem with the lock
        """
        state['_lock'] = Lock()
        self.__dict__.update(state)

    def registerAggregator(self, fieldname, methodname):
        if self._reg_ag.get(fieldname):
            raise KeyError('Aggregator for field "%s" already registered' % fieldname)
        self._reg_ag.register(fieldname, methodname)


    def registerDisaggregator(self, fieldname, methodname):
        if self._reg_dag.get(fieldname):
            raise KeyError('Disaggregator for field "%s" already registered' % fieldname)
        self._reg_dag.register(fieldname, methodname)

    def get(self, name, instance, **kwargs):
        methodname = self._reg_ag.get(name)
        if not methodname:
            raise KeyError('No aggregator registered for field "%s"' % name)
        method = getattr(instance, methodname)
        if not method:
            raise KeyError('Aggregator "%s" for field "%s" not found' % (methodname, name))
        result = method(name, instance, **kwargs)
        if not isinstance(result, DictType):
            raise TypeError('Result returned from an aggregator must be DictType')
        return result[name]

        if self._caching:
            cache_entry = self._cache_get(instance.getId(), methodname)
        else:
            cache_entry = None

        if cache_entry is None:
            method = getattr(instance, methodname)
            if not method:
                raise KeyError('Aggregator "%s" for field "%s" not found' % (methodname, name))
            result = method(name, instance, **kwargs)
            if not isinstance(result, DictType):
                raise TypeError('Result returned from an aggregator must be DictType')

            if self._caching:
                self._cache_put(instance.getId(), methodname, result)

            if not result.has_key(name):
                raise KeyError('result dictionary returned from "%s"'
                               ' does not contain an key for "%s"' % 
                               (methodname, name))
            return result[name]
        else:
            return cache_entry[name]
        
    def set(self, name, instance, value, **kwargs):
        methodname = self._reg_dag.get(name)
        if not methodname:
            raise KeyError('No disaggregator registered for field "%s"' % name)

        method = getattr(instance, methodname)
        if not method:
            raise KeyError('Disaggregator "%s" for field "%s" not found' % (methodname, name))
        if self._caching:
            self._cache_remove(instance.getId(), methodname)
        method(name, instance, value, **kwargs)

    def unset(self, name, instance, **kwargs):
        pass

    ######################################################################
    # A very basic cache implementation to cache the result dictionaries
    # returned by the aggregators
    ######################################################################

    def _cache_get(self, objId, methodname):
        """ retrieve the result dictionary for (objId, methodname) """
        self._lock.acquire()
        entry = self.cache.get((objId, methodname))
        if entry is None: 
            self._lock.release()
            return None
        if time.time() - entry[0] > CACHE_TIMEOUT: 
            del self.cache[(objId, methodname)]
            self._lock.release()
            return None
        self._lock.release()
        return entry[1]

    def _cache_put(self, objId, methodname, result):
        """ store (objId, methodname) : (current_time, result) in cache """
        self._lock.acquire()
        self.cache[(objId, methodname)] = (time.time(), result)
        self._lock.release()

    def _cache_remove(self, objId, methodname):
        """ remove (objId, methodname) from cache """
        
        self._lock.acquire()
        key = (objId, methodname)
        if self.cache.has_key(key):
            del self.cache[key] 
        self._lock.release()

from Registry import registerStorage
registerStorage(AggregatedStorage)



## Script (Python) "session_save_form"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=fieldName,default
##title=Return field value previously stored on session by session_save_form
##

from Products.CMFCore.utils import getToolByName

# Avoid implicitly creating a session if one doesn't exists
session = None
sdm = getToolByName(context, 'session_data_manager', None)

if sdm is not None:
    session = sdm.getSessionData(create=0)

if session is None:
    return default

id = context.getId()
session_data = None

if session.has_key(id):
    session_dic = session.get(id)
    if session_dic.has_key(fieldName):
        session_data = session_dic.get(fieldName)

return session_data or default



################################################################################
#
# Copyright (c) 2002-2005, Benjamin Saller <bcsaller@ideasuite.com>, and
#                              the respective authors. All rights reserved.
# For a list of Archetypes contributors see docs/CREDITS.txt.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice, this
#   list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
# * Neither the name of the author nor the names of its contributors may be used
#   to endorse or promote products derived from this software without specific
#   prior written permission.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
################################################################################
"""
"""

import re
import sys
import logging

from zope.tal import ndiff
from Globals import InitializeClass, package_home
from UserDict import UserDict
import ExtensionClass
from Acquisition import Implicit
from AccessControl import ClassSecurityInfo
from ZPublisher.BeforeTraverse import registerBeforeTraverse
from ZPublisher.HTTPRequest import HTTPRequest
from ZPublisher.HTTPResponse import HTTPResponse
from Persistence import Persistent

from Products.Archetypes.atapi import registerType
from Products.Archetypes.atapi import process_types
from Products.Archetypes.atapi import listTypes
from Products.Archetypes.atapi import BaseContent
from Products.Archetypes.config import PKG_NAME

logger = logging.getLogger('Archetypes')

PACKAGE_HOME = package_home(globals())

try:
    import Zope2
except ImportError:
    ZOPE28 = False
else:
    ZOPE28 = True

try:
    from OFS import subscribers
except ImportError:
    ZOPE29 = False
else:
    ZOPE29 = True


def gen_class(klass, schema=None):
    """generats and registers the klass
    """
    if schema is not None:
        klass.schema = schema.copy()
    registerType(klass, 'Archetypes')
    content_types, constructors, ftis = process_types(listTypes(), PKG_NAME)

def mkDummyInContext(klass, oid, context, schema=None):
    gen_class(klass, schema)
    dummy = klass(oid=oid).__of__(context)
    context._setObject(oid, dummy, suppress_events=True)
    dummy.initializeArchetype()
    return dummy

def makeContent( container, portal_type, id='document', **kw ):
    container.invokeFactory( type_name=portal_type, id=id )
    return getattr( container, id )

class Dummy(BaseContent):
    def Title(self):
        return 'title'

def normalize_html(s):
    s = re.sub(r"\s+", " ", s)
    s = re.sub(r"(?s)\s+<", "<", s)
    s = re.sub(r"(?s)>\s+", ">", s)
    return s

def nicerange(lo, hi):
    if hi <= lo+1:
        return str(lo+1)
    else:
        return "%d,%d" % (lo+1, hi)

def showdiff(a, b):
    cruncher = ndiff.SequenceMatcher(ndiff.IS_LINE_JUNK, a, b)
    for tag, alo, ahi, blo, bhi in cruncher.get_opcodes():
        if tag == "equal":
            continue
        print nicerange(alo, ahi) + tag[0] + nicerange(blo, bhi)
        ndiff.dump('<', a, alo, ahi)
        if a and b:
            print '---'
        ndiff.dump('>', b, blo, bhi)

def populateFolder(folder, folder_type, doc_type):
    """ Creates a structure like:

    \index_html
    \doc1
    \folder1
       \folder11
       \folder12
       \doc11
    \folder2
       \folder21
       \doc21
       \index_html
       \folder22
          \folder221
             \doc2211
             \doc2212
          \folder222
             \doc2221
             \doc2222
          \folder223
             \doc2231
             \doc2232
    """
    folder.invokeFactory(doc_type, id='index_html')
    folder.invokeFactory(doc_type, id='doc1')
    folder.invokeFactory(folder_type, id='folder1')
    folder.invokeFactory(folder_type, id='folder2')
    f1 = folder.folder1
    f2 = folder.folder2
    f1.invokeFactory(folder_type, id='folder11')
    f1.invokeFactory(folder_type, id='folder12')
    f1.invokeFactory(doc_type, id='doc11')
    f2.invokeFactory(folder_type, id='folder21')
    f2.invokeFactory(folder_type, id='folder22')
    f2.invokeFactory(doc_type, id='doc21')
    f2.invokeFactory(doc_type, id='index_html')
    f22 = f2.folder22
    f22.invokeFactory(folder_type, id='folder221')
    f221 = f22.folder221
    f221.invokeFactory(doc_type, id='doc2211')
    f221.invokeFactory(doc_type, id='doc2212')
    f22.invokeFactory(folder_type, id='folder222')
    f222 = f22.folder222
    f222.invokeFactory(doc_type, id='doc2221')
    f222.invokeFactory(doc_type, id='doc2222')
    f22.invokeFactory(folder_type, id='folder223')
    f223 = f22.folder223
    f223.invokeFactory(doc_type, id='doc2231')
    f223.invokeFactory(doc_type, id='doc2232')

WRAPPER = '__at_is_wrapper_method__'
ORIG_NAME = '__at_original_method_name__'
def isWrapperMethod(meth):
    return getattr(meth, WRAPPER, False)

def wrap_method(klass, name, method, pattern='__at_wrapped_%s__'):
    old_method = getattr(klass, name)
    if isWrapperMethod(old_method):
        logger.warning('Wrapping already wrapped method at %s.%s' %
                       (klass.__name__, name))
    new_name = pattern % name
    setattr(klass, new_name, old_method)
    setattr(method, ORIG_NAME, new_name)
    setattr(method, WRAPPER, True)
    setattr(klass, name, method)

def unwrap_method(klass, name):
    old_method = getattr(klass, name)
    if not isWrapperMethod(old_method):
        raise ValueError, ('Trying to unwrap non-wrapped '
                           'method at %s.%s' % (klass.__name__, name))
    orig_name = getattr(old_method, ORIG_NAME)
    new_method = getattr(klass, orig_name)
    delattr(klass, orig_name)
    setattr(klass, name, new_method)

class FakeRequestSession(ExtensionClass.Base, UserDict):
    """Dummy dict like object with set method for SESSION and REQUEST
    """
    security = ClassSecurityInfo()
    security.setDefaultAccess('allow')
    security.declareObjectPublic()
    
    def __init__(self):
        UserDict.__init__(self)
        # add a dummy because request mustn't be empty for test
        # like 'if REQUEST:'
        self['__dummy__'] = None
    
    def __nonzero__(self):
        return True
    
    def set(self, key, value):
        self[key] = value

InitializeClass(FakeRequestSession)
FakeRequestSession()

class DummySessionDataManager(Implicit):
    """Dummy sdm for sessioning
    
    Uses traversal hooks to add the SESSION object as lazy item to the request
    """
    security = ClassSecurityInfo()
    security.setDefaultAccess('allow')
    security.declareObjectPublic()
    
    id = 'session_data_manager'
    
    def __init__(self):
        self.session = FakeRequestSession()
        
    def manage_afterAdd(self, item, container):
        """Register traversal hooks to add SESSION to request
        """
        parent = self.aq_inner.aq_parent
        hook = DummySDMTraverseHook()
        registerBeforeTraverse(parent, hook, 'SessionDataManager', 50)

    def getSessionData(self, create=1):
        """ """
        return self.session
    
    def hasSessionData(self):
        """ """
        return True
    
    def getSessionDataByKey(self, key):
        """ """
        return self.session
    
    def getBrowserIdManager(self):
        """ """
        # dummy
        return self

InitializeClass(DummySessionDataManager)

class DummySDMTraverseHook(Persistent):
    """Traversal hook for dummy sessions
    
    See Products.Sessions.SessionDataManager.SessionDataManagerTraverser
    """
    
    def __call__(self, container, request):
        id = DummySessionDataManager.id
        sdm = getattr(container, id)
        getSessionData = sdm.getSessionData
        request.set_lazy('SESSION', getSessionData)

from Products.Five import BrowserView
class SimpleView(BrowserView):
    def __call__(self):
        return 'SimpleView simple output'

def aputrequest(file, content_type):
    resp = HTTPResponse(stdout=sys.stdout)
    environ = {}
    environ['SERVER_NAME']='foo'
    environ['SERVER_PORT']='80'
    environ['REQUEST_METHOD'] = 'PUT'
    environ['CONTENT_TYPE'] = content_type
    req = HTTPRequest(stdin=file, environ=environ, response=resp)
    return req
